package format

import (
	"git.gammaspectra.live/S.O.N.G/Kirika/audio"
	"io"
)

type Format interface {
	// Identify checks whether a format is of a type. peek includes a few first bytes, extension is the lowercase file extension without a dot.
	Identify(peek []byte, extension string) bool
}

type WriteSeekCloser interface {
	io.Writer
	io.Closer
	io.Seeker
}

type Decoder interface {
	Format
	// Open a stream and decodes it into an audio.Source
	Open(r io.ReadSeekCloser) (audio.Source, error)
}

type Encoder interface {
	Format
	// Encode Receives an audio.Source and encodes it into a writer. Some formats can do special operations if writer is also an io.Seeker
	Encode(source audio.Source, writer io.WriteCloser, options map[string]interface{}) error
}
