package mp3

/*
#cgo CFLAGS: -I"${SRCDIR}/../../../cgo" -march=native -Ofast -std=c99
#include "audio.h"
*/
import "C"
import (
	"errors"
	"fmt"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio"
	mp3Lib "github.com/kvark128/minimp3"
	"github.com/viert/go-lame"
	"io"
	"unsafe"
)

const BlockSize = 1024 * 128

type Format struct {
}

func NewFormat() Format {
	return Format{}
}

func (f Format) Open(r io.ReadSeekCloser) (audio.Source, error) {
	decoder := mp3Lib.NewDecoder(r)

	_, err := decoder.Read([]byte{})
	if err != nil {
		return audio.Source{}, err
	}

	newChannel := make(chan []float32)

	go func() {
		defer close(newChannel)
		samples := make([]int16, BlockSize*2)
		const SizeofInt16 = int(unsafe.Sizeof(int16(0)))
		byteSlice := unsafe.Slice((*byte)(unsafe.Pointer(&samples[0])), len(samples)*SizeofInt16)
		for {
			n, err := decoder.Read(byteSlice)
			if err != nil {
				return
			}
			n /= SizeofInt16

			buf := make([]float32, n)

			//convert 16-bit to f32 samples
			C.audio_int16_to_float32((*C.int16_t)(&samples[0]), C.size_t(n), (*C.float)(&buf[0]), C.int(16))

			newChannel <- buf
		}
	}()

	return audio.Source{
		Channels:   decoder.Channels(),
		SampleRate: decoder.SampleRate(),
		Blocks:     newChannel,
	}, nil
}

func (f Format) Encode(source audio.Source, writer io.WriteCloser, options map[string]interface{}) error {
	var vbr = true
	var bitrate = 0

	if options != nil {
		var val interface{}
		var ok bool
		var intVal int
		var strVal string
		if val, ok = options["bitrate"]; ok {
			if strVal, ok = val.(string); ok {
				switch strVal {
				case "v0":
					vbr = true
					bitrate = 0
				case "v1":
					vbr = true
					bitrate = 1
				case "v2":
					vbr = true
					bitrate = 2
				case "v3":
					vbr = true
					bitrate = 3
				case "320k":
					vbr = false
					bitrate = 320
				case "256k":
					vbr = false
					bitrate = 256
				case "192k":
					vbr = false
					bitrate = 192
				case "128k":
					vbr = false
					bitrate = 128

				default:
					return fmt.Errorf("unknown setting bitrate=%s", strVal)
				}
			} else if intVal, ok = val.(int); ok {
				vbr = false
				bitrate = intVal
			}
		}
	}

	encoder := lame.NewEncoder(writer)

	if encoder == nil {
		return errors.New("could not create encoder")
	}
	defer encoder.Close()
	if err := encoder.SetNumChannels(source.Channels); err != nil {
		return err
	}
	if err := encoder.SetInSamplerate(source.SampleRate); err != nil {
		return err
	}
	encoder.SetWriteID3TagAutomatic(false)

	if vbr {
		if err := encoder.SetVBR(lame.VBRMTRH); err != nil {
			return err
		}
		if err := encoder.SetVBRQuality(float64(bitrate)); err != nil {
			return err
		}
	} else {
		if err := encoder.SetVBR(lame.VBROff); err != nil {
			return err
		}
		if err := encoder.SetBrate(bitrate); err != nil {
			return err
		}
	}

	for block := range source.Blocks {
		samples := make([]int16, len(block))

		C.audio_float32_to_int16((*C.float)(&block[0]), C.size_t(len(block)), (*C.int16_t)(&samples[0]))

		_, err := encoder.Write(unsafe.Slice((*byte)(unsafe.Pointer(&samples[0])), len(samples)*2))

		if err != nil {
			return err
		}
	}

	return nil
}

func (f Format) Identify(peek []byte, extension string) bool {
	return /*bytes.Compare(peek[:4], []byte{'f', 'L', 'a', 'C'}) == 0 || */ extension == "mp3"
}
