package opus

/*
#cgo CFLAGS: -I"${SRCDIR}/../../../cgo" -march=native -Ofast -std=c99
#include "audio.h"
*/
import "C"
import (
	"bytes"
	"fmt"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio"
	libopus "git.gammaspectra.live/S.O.N.G/go-pus"
	"io"
)

const FixedSampleRate int = 48000

type Format struct {
}

func NewFormat() Format {
	return Format{}
}

func (f Format) Open(r io.ReadSeekCloser) (audio.Source, error) {
	stream, err := libopus.NewStream(r)
	if err != nil {
		return audio.Source{}, err
	}

	newChannel := make(chan []float32)

	go func() {
		defer stream.Close()
		defer close(newChannel)

		buf := make([]int16, 120*FixedSampleRate*2)

		for {

			n, err := stream.ReadStereo(buf)
			if err != nil {
				return
			}

			buffer := make([]float32, n*2)
			C.audio_int16_to_float32((*C.int16_t)(&buf[0]), C.size_t(n*2), (*C.float)(&buffer[0]), C.int(16))

			if n > 0 {
				newChannel <- buffer
			}
		}
	}()

	//We always get two channels via stereo api
	return audio.Source{
		Channels:   2,
		SampleRate: FixedSampleRate,
		Blocks:     newChannel,
	}, nil
}

func (f Format) Encode(source audio.Source, writer io.WriteCloser, options map[string]interface{}) error {
	if source.SampleRate != FixedSampleRate {
		//TODO: maybe not needed
		return fmt.Errorf("invalid source SampleRate: expected %d, got %d", FixedSampleRate, source.SampleRate)
	}

	var bitrate = 0

	if options != nil {
		var val interface{}
		var ok bool
		var intVal int
		var strVal string
		if val, ok = options["bitrate"]; ok {
			if strVal, ok = val.(string); ok {
				switch strVal {
				case "auto":
					bitrate = 0
				case "256k":
					bitrate = 256
				case "192k":
					bitrate = 192
				case "128k":
					bitrate = 128

				default:
					return fmt.Errorf("unknown setting bitrate=%s", strVal)
				}
			} else if intVal, ok = val.(int); ok {
				bitrate = intVal
			}
		}
	}

	encoder, err := libopus.NewEncoder(FixedSampleRate, source.Channels, libopus.AppAudio, writer)

	if err != nil {
		return err
	}
	defer encoder.Close()

	if bitrate == 0 {
		err = encoder.SetBitrateToAuto()

		if err != nil {
			return err
		}
	} else {
		//kbits to bits
		err = encoder.SetBitrate(bitrate * 1024)

		if err != nil {
			return err
		}
	}

	for block := range source.Blocks {

		samples := make([]int16, len(block))
		C.audio_float32_to_int16((*C.float)(&block[0]), C.size_t(len(block)), (*C.int16_t)(&samples[0]))

		err = encoder.Encode(samples)
		if err != nil {
			return err
		}
	}

	return nil

}

func (f Format) Identify(peek []byte, extension string) bool {
	return bytes.Compare(peek[:4], []byte{'O', 'g', 'g', 'S'}) == 0 || extension == "opus"
}
