package packetizer

import (
	"bufio"
	aac_adts "github.com/edgeware/mp4ff/aac"
	"io"
)

type AdtsPacketizer struct {
	reader  io.Reader
	samples int64
}

type AdtsPacket struct {
	sampleNumber int64
	header       *aac_adts.ADTSHeader
	data         []byte
}

func (p *AdtsPacket) KeepMode() KeepMode {
	return Discard
}

func (p *AdtsPacket) GetSampleNumber() int64 {
	return p.sampleNumber
}

func (p *AdtsPacket) GetData() []byte {
	return append(p.header.Encode(), p.data...)
}

func NewAdtsPacketizer(reader io.Reader) *AdtsPacketizer {
	return &AdtsPacketizer{reader: bufio.NewReader(reader)}
}

func (o *AdtsPacketizer) GetPacket() Packet {
	packet := &AdtsPacket{}

	header, _, err := aac_adts.DecodeADTSHeader(o.reader)

	if err != nil {
		return nil
	}

	packet.header = header
	packet.data = make([]byte, packet.header.PayloadLength)
	if _, err := o.reader.Read(packet.data); err != nil {
		return nil
	}
	packet.sampleNumber = o.samples
	o.samples += 1024 //always this fixed size per frame

	return packet
}
