package packetizer

import (
	"bufio"
	mp3parser "github.com/sssgun/mp3"
	"io"
)

type Mp3Packetizer struct {
	decoder *mp3parser.Decoder
	samples int64
}

type Mp3Packet struct {
	sampleNumber int64
	frame        mp3parser.Frame
}

func (p *Mp3Packet) KeepMode() KeepMode {
	return Discard
}

func (p *Mp3Packet) GetSampleNumber() int64 {
	return p.sampleNumber
}

func (p *Mp3Packet) GetData() []byte {
	return p.frame.Bytes()
}

func NewMp3Packetizer(reader io.Reader) *Mp3Packetizer {
	return &Mp3Packetizer{decoder: mp3parser.NewDecoder(bufio.NewReader(reader))}
}

func (o *Mp3Packetizer) GetPacket() Packet {
	packet := &Mp3Packet{}

	var skipped int

	if err := o.decoder.Decode(&packet.frame, &skipped); err != nil {
		return nil
	}

	packet.sampleNumber = o.samples
	o.samples += int64(packet.frame.Samples())

	return packet
}
