package packetizer

import (
	"bufio"
	"bytes"
	"encoding/binary"
	"io"
)

type OggPacketizer struct {
	reader io.Reader
}

type OggPacket struct {
	header struct {
		Capture                 [4]byte
		Revision                byte
		Flags                   byte
		AbsoluteGranulePosition int64
		StreamSerialNumber      uint32
		PageSequenceNumber      uint32
		PageChecksum            uint32
		PageSegments            byte
	}
	pageSegmentTable []byte
	segments         [][]byte
}

func (p *OggPacket) KeepMode() KeepMode {
	if p.header.AbsoluteGranulePosition == 0 {
		return Keep
	}
	return Discard
}

func (p *OggPacket) GetSampleNumber() int64 {
	//TODO
	return p.header.AbsoluteGranulePosition
}

func (p *OggPacket) GetData() []byte {
	buf := bytes.NewBuffer(make([]byte, 0, 4+1+1+8+4+4+4+1+int(p.header.PageSegments)*(255+1)))
	binary.Write(buf, binary.LittleEndian, p.header)
	binary.Write(buf, binary.LittleEndian, p.pageSegmentTable)
	for _, segment := range p.segments {
		if len(segment) > 0 {
			binary.Write(buf, binary.LittleEndian, segment)
		}
	}

	return buf.Bytes()
}

func NewOggPacketizer(reader io.Reader) *OggPacketizer {
	return &OggPacketizer{reader: bufio.NewReader(reader)}
}

func (o *OggPacketizer) GetPacket() Packet {
	packet := &OggPacket{}
	if err := binary.Read(o.reader, binary.LittleEndian, &packet.header); err != nil || bytes.Compare(packet.header.Capture[:], []byte{'O', 'g', 'g', 'S'}) != 0 {
		return nil
	}

	packet.pageSegmentTable = make([]byte, packet.header.PageSegments)
	if err := binary.Read(o.reader, binary.LittleEndian, packet.pageSegmentTable); err != nil {
		return nil
	}
	packet.segments = make([][]byte, packet.header.PageSegments)

	for i, size := range packet.pageSegmentTable {
		if size > 0 {
			packet.segments[i] = make([]byte, size)
			if err := binary.Read(o.reader, binary.LittleEndian, packet.segments[i]); err != nil {
				return nil
			}
		}
	}

	return packet
}
